@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 #
 #@file device.py
-#@brief device descripto
+#@brief device descriptor
 #@author Jason Berger
 #@date 02/19/2019
 #
@@ -10,6 +10,7 @@ import sys
 import os
 import yaml
 import json
+import copy
 from mrtutils.mrtYamlHelper import *
 from mrtutils.mrtTemplateHelper import *
 
@@ -136,7 +137,7 @@ class DevConfig:
 
 
 
-class FieldVal:
+class FieldValSpec:
     def __init__(self, node):
         self.field = 0
         self.val = 0 
@@ -152,14 +153,14 @@ class FieldVal:
             self.desc = valItem['name']
 
     def getFieldValMacro(self, spacing = 0):
-        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.name.upper()+"_"+self.field.name.upper() +"_" + self.name.upper()
+        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.pattern.replace('$n','X').upper()+"_"+self.field.name.upper() +"_" + self.name.upper()
         if spacing > 0:
             spaces = spacing - len(ret)
             ret = ret + (" " * spaces)
         return ret
     
     def getFieldValMaskMacro(self, spacing = 0):
-        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.name.upper()+"_"+self.field.name.upper() +"_" + self.name.upper() + "_MASK"
+        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.pattern.replace('$n','X').upper()+"_"+self.field.name.upper() +"_" + self.name.upper() + "_MASK"
         if spacing > 0:
             spaces = spacing - len(ret)
             ret = ret + (" " * spaces)
@@ -179,7 +180,7 @@ class FieldVal:
             return "b"+format(self.val, '0'+str(self.field.bitCount)+'b')
 
  
-class RegField:
+class RegFieldSpec:
     def __init__(self, node):
         self.reg = 0
         self.mask = 0XFFFFFFFF
@@ -199,21 +200,21 @@ class RegField:
             self.mask = 1 << node['bit']
         if 'values' in node:
             for valNode in node['values']:
-                newVal = FieldVal(valNode)
+                newVal = FieldValSpec(valNode)
                 self.addVal(newVal)
         if 'vals' in node:
             for valNode in node['vals']:
-                newVal = FieldVal(valNode)
+                newVal = FieldValSpec(valNode)
                 self.addVal(newVal)
         if 'desc' in node:
             self.desc = node['desc']
         
-        if self.getSize() == 1:
+        if (self.getSize() == 1) and (len(self.vals) == 0):
             self.isFlag = True
         
         self.bitCount = self.getSize()
         self.offset = self.getOffset()
-        self.startBit = self.bitCount + self.offset
+        self.startBit = self.bitCount + self.offset -1
 
     def getSize(self):
         count = 0
@@ -238,7 +239,7 @@ class RegField:
         self.valDict[fieldVal.name] = fieldVal
 
     def getFieldMaskMacro(self, spacing = 0):
-        ret = self.reg.device.prefix.upper() +"_"+self.reg.name.upper()+"_"+self.name.upper() +"_FIELD_MASK"
+        ret = self.reg.device.prefix.upper() +"_"+self.reg.pattern.replace('$n','X').upper()+"_"+self.name.upper() +"_FIELD_MASK"
 
         if spacing > 0:
             spaces = spacing - len(ret)
@@ -246,7 +247,7 @@ class RegField:
         return ret
     
     def getFieldOffsetkMacro(self, spacing = 0):
-        ret = self.reg.device.prefix.upper() +"_"+self.reg.name.upper()+"_"+self.name.upper() +"_FIELD_OFFSET"
+        ret = self.reg.device.prefix.upper() +"_"+self.reg.pattern.replace('$n','X').upper()+"_"+self.name.upper() +"_FIELD_OFFSET"
 
         if spacing > 0:
             spaces = spacing - len(ret)
@@ -254,7 +255,7 @@ class RegField:
         return ret
 
     def getFieldFlagMacro(self, spacing = 0):
-        ret = self.reg.device.prefix.upper() +"_"+ self.reg.name.upper()+"_"+self.name.upper()
+        ret = self.reg.device.prefix.upper() +"_"+ self.reg.pattern.replace('$n','X').upper()+"_"+self.name.upper()
 
         if spacing > 0:
             spaces = spacing - len(ret)
@@ -295,7 +296,7 @@ class RegField:
         lDesc +=2
 
         divLine = "+" + ("-"* lName) + "+" + ("-"* lVal) + "+" + ("-"* lDesc)  + "+"
-        titleLine = "|" + t.padAfter("Name", lName) + "|" + t.padAfter("Address", lVal) + "|" + t.padAfter("Description", lDesc)  + "|"
+        titleLine = "|" + t.padAfter("Name", lName) + "|" + t.padAfter("Value", lVal) + "|" + t.padAfter("Description", lDesc)  + "|"
         headLine = divLine.replace("-","=")
 
         lines.append(divLine)
@@ -322,7 +323,7 @@ class RegField:
     
         
 
-class DeviceReg:
+class DeviceRegSpec:
     def __init__(self,name):
         self.name = name
         self.addr = 0
@@ -339,6 +340,9 @@ class DeviceReg:
         self.hasFlags = False
         self.hasFields = False
         self.configs = {}
+        self.pattern = ""
+        self.nIdx = 0
+        self.descPattern = ""
     
     def addField(self, field):
         field.reg = self
@@ -364,7 +368,7 @@ class DeviceReg:
         if self.size == 4: 
             fieldNode['mask'] = 0xFFFFFFFF
 
-        field = RegField(fieldNode)
+        field = RegFieldSpec(fieldNode)
         field.flat = True
         self.addField(field)
     
@@ -441,6 +445,7 @@ class DeviceReg:
             if len(field.name) > cellWidth:
                 cellWidth = len(field.name)
         
+        
         lines = ["+------------","|Bit         ", "+============", "| **Field**  ","+------------"]
 
         for i in range(bits,-1,-1):
@@ -487,7 +492,7 @@ class DeviceReg:
 
 
 
-class Device:
+class DeviceSpec:
     def __init__(self, name):
         self.name = name
         self.prefix = ""
@@ -505,6 +510,8 @@ class Device:
         self.defaults = {}
         self.nextAddr = 0
         self.storageSize = 0
+        self.slave = False
+        self.consolidateDuplicates = False
 
     def addReg(self, reg):
         reg.device = self
@@ -600,8 +607,13 @@ class Device:
         return out
 
     def parseYAML(self,yamlFile):
-        data = open(yamlFile)
-        objDevice = yaml.load(data , Loader=yaml.FullLoader)
+
+        if type(yamlFile) is str:
+
+            data = open(yamlFile)
+            objDevice = yaml.load(data , Loader=yaml.FullLoader)
+        else:
+            objDevice = yamlFile
 
         if 'name' in objDevice:
             self.name = objDevice['name']
@@ -640,48 +652,98 @@ class Device:
 
         if 'registers' in objDevice:
             regs = yamlNormalizeNodes( objDevice['registers'], 'reg_name','addr')
-            for regItem in regs:
-                newReg = DeviceReg(regItem['reg_name'])
-                
-                if 'type' in regItem:
-                    newReg.type = regItem['type'].replace("_t","")
-                    newReg.size = sizeDict[newReg.type.replace("_t","")]
-                if 'size' in regItem:
-                    newReg.size = regItem['size']
-                if 'perm' in regItem:
-                    newReg.perm = regItem['perm'].upper()
-                if 'desc' in regItem:
-                    newReg.desc = regItem['desc']
-                if 'name' in regItem:
-                    newReg.desc = regItem['name']
-                if 'default' in regItem:
-                    newReg.default = regItem['default']
-                    newReg.hasDefault = True
-                
-                
-                if 'addr' in regItem:
-                    newReg.addr = regItem['addr'] #int(regItem['addr'],0) 
+            for regItemRaw in regs:
+
+                count = 1 
+                start = 0
+                expandedRegs = [] 
+
+
+                #If reg name has an expansion pattern '$n' create multiple instances
+                if '$n' in regItemRaw['reg_name']:
+
+                    if 'count' in regItemRaw:
+                        count = regItemRaw['count']
+
+                    if 'n' in regItemRaw:
+                        start = regItemRaw['n']
+
+
+                    for i in range(start, count ):
+                        expandedItem = copy.deepcopy(regItemRaw)
+                        expandedItem['reg_name'] = expandedItem['reg_name'].replace('$n', str(i))
+                        expandedItem['descPattern'] = expandedItem['desc'];
+                        expandedItem['desc'] = expandedItem['desc'].replace('$n', str(i))
+                        expandedItem['pattern'] = regItemRaw['reg_name']
+                        expandedRegs.append(expandedItem)
+
                 else:
-                    newReg.addr = self.nextAddr
+                    expandedRegs.append(regItemRaw)
+                    
+                for idx, regItem in enumerate(expandedRegs): 
+                    newReg = DeviceRegSpec(regItem['reg_name'])
+                    newReg.nIdx = idx
+                    
+                    if 'type' in regItem:
+                        newReg.type = regItem['type'].replace("_t","")
+                        newReg.size = sizeDict[newReg.type.replace("_t","")]
+                    if 'size' in regItem:
+                        newReg.size = regItem['size']
+                    if 'perm' in regItem:
+                        newReg.perm = regItem['perm'].upper()
+                    if 'desc' in regItem:
+                        newReg.desc = regItem['desc']
+                    if 'name' in regItem:
+                        newReg.desc = regItem['name']
+                    if 'pattern' in regItem:
+                        newReg.pattern = regItem['pattern']
+                    if 'descPattern' in regItem:
+                        newReg.descPattern = regItem['descPattern']
+                    if 'default' in regItem:
+                        newReg.default = regItem['default']
+                        newReg.hasDefault = True
+                    
+                    
+                    
+                    if 'addr' in regItem:
+                        newReg.addr = regItem['addr'] #int(regItem['addr'],0) 
+                    else:
+                        newReg.addr = self.nextAddr
 
-                self.nextAddr = newReg.addr +  newReg.size
+                    self.nextAddr = newReg.addr +  newReg.size
 
-                self.storageSize += newReg.size
-                self.addReg(newReg)      
+                    self.storageSize += newReg.size
+                    self.addReg(newReg)      
         
         if 'fields' in objDevice:
             for propNode in objDevice['fields']:
                 if type(propNode) is dict:
                     regName = list(propNode.keys())[0]
                     propItem = list(propNode.values())[0]
+
+                    fieldNodes = yamlNormalizeNodes(propItem, 'name', 'mask')
+
+                    if '$n' in regName:
+                        for key in self.regs:   
+                            if self.regs[key].pattern == regName:
+                                for fieldNode in fieldNodes:
+                                    newField = RegFieldSpec(fieldNode)
+                                    self.regs[key].addField(newField)
+
+
                     if regName in self.regs:
                         curReg = self.regs[regName]  
-                        fieldNodes = yamlNormalizeNodes(propItem, 'name', 'mask')
                         for fieldNode in fieldNodes:
-                            newField = RegField(fieldNode)
+                            newField = RegFieldSpec(fieldNode)
                             curReg.addField(newField)
                 else:
                     regName = propNode
+
+                    if '$n' in regName:
+                        for key in self.regs:   
+                            if self.regs[key].pattern == regName:
+                                self.regs[key].addDefaultField()
+
                     if regName in self.regs:
                         self.regs[regName].addDefaultField()
         
