@@ -16,6 +16,17 @@ import urllib.request
 from mrtutils.mrtYamlHelper import *
 import xml.etree.ElementTree as ET 
 
+class bcolors:
+    HEADER = '\033[95m'
+    OKBLUE = '\033[94m'
+    OKCYAN = '\033[96m'
+    OKGREEN = '\033[92m'
+    WARNING = '\033[93m'
+    FAIL = '\033[91m'
+    ENDC = '\033[0m'
+    BOLD = '\033[1m'
+    UNDERLINE = '\033[4m'
+
 
 sizeDict = {
     "uint8" : 1,
@@ -106,8 +117,12 @@ class GattValue:
         self.desc = None
     
     def parseYaml(self, node):
+
         yamlGetAttributes(node, attrDict, self)
 
+        if self.desc == None:
+            self.desc = self.name
+
     def getDict(self):
 
         json_dict = { "name": self.name, "val": self.value }
@@ -128,7 +143,7 @@ class GattCharacteristic(object):
         self.perm = None
         self.url =""
         self.uuid = None
-        self.uuidType = "e128Bit"
+        self.uuidType = "MRT_UUID_LEN_128"
         self.icon = ''
         self.coef = 1
         self.unit = ''
@@ -142,7 +157,7 @@ class GattCharacteristic(object):
         arr =[]
         if type(val) != str:
             val = "%0.4X" % val
-            self.uuidType ="e16Bit"
+            self.uuidType ="MRT_UUID_LEN_16"
         
         val = val.replace('-','')
         arr = [val[i:i+2] for i in range(0, len(val), 2)]
@@ -235,6 +250,8 @@ class GattCharacteristic(object):
 
         yamlGetAttributes(node, attrDict, self)
 
+        self.type = self.type.replace("_t", "")
+
         if self.type in ['flag','flags','mask','bits']:
             self.isMask = True
 
@@ -246,6 +263,7 @@ class GattCharacteristic(object):
             if(m.group(1) != ''):
                 self.arrayLen = int(m.group(1))
             self.type = self.type[0:m.start()]
+        
 
         if 'vals' in node:
             valNodes = yamlNormalizeNodes( node['vals'], 'name','desc')
@@ -309,15 +327,18 @@ class GattService(object):
         #load uri first so we can override attributes as needed 
         
         self.name = "unnamed"
+        self.prefix = ""
         self.chars = []
         self.desc = None
         self.isStandard = False
         self.url =""
         self.uri = None
         self.nextUuid = 0
-        self.uuidType ="e128Bit"
+        self.uuidType ="MRT_UUID_LEN_128"
         self.profile = ''
         self.icon = ''
+        self.attrCount = 0
+        
     
     def loadUri(self, uri):
         print("Pulling " + uri)
@@ -328,7 +349,7 @@ class GattService(object):
 
         self.name = root.attrib['name'].replace(' ', '_')
         self.uuid = int(root.attrib['uuid'],16)
-        self.uuidType = 'e16Bit'
+        self.uuidType = 'MRT_UUID_LEN_16'
         setIfEmpty(self, 'desc',root.find('./InformativeText/Abstract'))
         setIfEmpty(self, 'desc',root.find('./InformativeText/Summary'))
 
@@ -348,7 +369,7 @@ class GattService(object):
         arr =[]
         if type(val) != str:
             val = "%0.4X" % val
-            self.uuidType ="e16Bit"
+            self.uuidType ="MRT_UUID_LEN_16"
         
         val = val.replace('-','')
         arr = [val[i:i+2] for i in range(0, len(val), 2)]
@@ -369,6 +390,11 @@ class GattService(object):
             yamlGetAttributes(node, attrDict, self)
             uuidSplit = self.uuid.split('-')
             self.nextUuid = int(uuidSplit[1], 16) + 1
+
+        
+
+
+        self.prefix = self.prefix.lower()
         
         if 'chars' in node:
             chars = node['chars']
@@ -391,6 +417,18 @@ class GattService(object):
     def addChar(self, char):
         self.chars.append(char)
 
+    def getAttrCount(self):
+
+        count = 1 # #1 attr for the service itself
+
+        for char in self.chars:
+            count +=2 #each char has a declaration and value attribute 
+
+            if 'n' in char.perm.lower():
+                count+=1        #each char gets an additional attribute for CCCD if it has notify permission
+        
+        return count
+
     def getDict(self):
         char_arr = []
 
@@ -400,6 +438,18 @@ class GattService(object):
         json_dict = { "name": self.name, "id": self.name.replace(' ', '_'), "uuid": uuidStr(self.uuid),  "short_uuid": uuidStr(self.uuid, True), "uuid_type": self.uuidType, "desc": self.desc, "icon" : self.icon, "uri": self.uri, "url": self.url, "characteristics": char_arr}
 
         return json_dict
+
+    def validate(self):
+
+        
+        if self.prefix == "":
+            self.prefix = self.name[0:3].lower() 
+            print( bcolors.WARNING + "Prefix not provided for '{0}' Service, defaulting to '{1}'".format(self.name, self.prefix)+ bcolors.ENDC)
+
+        for adx, chr in enumerate(self.chars):
+            if chr.perm == None:
+                chr.perm = "R"
+                print(bcolors.WARNING+"Permissions not provided for {0} Characteristic, defaulting to Read only".format(chr.name) + bcolors.ENDC)
              
 
 class GattProfile(object):
@@ -407,6 +457,9 @@ class GattProfile(object):
         self.name= "unnamed"
         self.services = []
         self.genTime = datetime.datetime.now().strftime("%m/%d/%y")
+        self.platform = "mrt"
+        self.attrCount = 0
+
     
     def nrfServices(self, uuidType):
 
@@ -446,11 +499,43 @@ class GattProfile(object):
         for serviceNode in serviceNodes:
             newService = GattService()
             newService.parseYaml(serviceNode)
+            print("Created " + newService.name)
             self.addService(newService)
+    
+    def validate(self):
+
+        names = [] 
+        pres = []
+
+        for idx,svc in enumerate(self.services):
+
+            if svc.name in names: 
+                for name in names:
+                    if name == svc.name:
+                        print(bcolors.WARNING+"Duplicate Service names: {0}".format(svc.name)+ bcolors.ENDC)
+                        return False
+
+            if svc.prefix in pres:
+                    for adx,pre in enumerate(pres):
+                        if pre == svc.prefix:
+                            print(bcolors.WARNING+"Duplicate Service prefix: \n{0} -> {1}\n{0} -> {1}".format(svc.name, svc.prefix, names[adx], svc.prefix)+ bcolors.ENDC)
+                            return False
+            
+            names.append(svc.name)
+            pres.append(svc.prefix)
+
+
+        if self.platform == 'esp32' and self.attrCount > 100:
+            print(bcolors.FAIL+"WARNING: {0} contains {1} attributes. Some Versions of the ESP-IDF have a limit of 100 attributes".format(self.name, self.attrCount) + bcolors.FAIL)
+        
+        return True
 
     def addService(self, service):
+
+        service.validate()
         service.profile = self
         self.services.append(service)
+        self.attrCount += service.getAttrCount()
     
     def getDict(self):
         svc_arr = []
